@@ -1,4 +1,4 @@
-/* $Id: hparser.c,v 2.53 2001/03/10 04:25:57 gisle Exp $
+/* $Id: hparser.c,v 2.71 2001/04/27 14:54:09 gisle Exp $
  *
  * Copyright 1999-2001, Gisle Aas
  * Copyright 1999-2000, Michael A. Chase
@@ -19,14 +19,16 @@ static
 struct literal_tag {
     int len;
     char* str;
+    int is_cdata;
 }
 literal_mode_elem[] =
 {
-    {6, "script"},
-    {5, "style"},
-    {3, "xmp"},
-    {9, "plaintext"},
-    {0, 0}
+    {6, "script", 1},
+    {5, "style", 1},
+    {3, "xmp", 1},
+    {9, "plaintext", 1},
+    {8, "textarea", 0},
+    {0, 0, 0}
 };
 
 enum argcode {
@@ -35,16 +37,23 @@ enum argcode {
     ARG_TOKENPOS,
     ARG_TOKEN0,
     ARG_TAGNAME,
+    ARG_TAG,
     ARG_ATTR,
+    ARG_ATTRARR,
     ARG_ATTRSEQ,
     ARG_TEXT,
     ARG_DTEXT,
     ARG_IS_CDATA,
     ARG_OFFSET,
     ARG_LENGTH,
+    ARG_LINE,
+    ARG_COLUMN,
     ARG_EVENT,
     ARG_UNDEF,
-    ARG_LITERAL /* Always keep last */
+    ARG_LITERAL, /* Always keep last */
+
+    /* extra flags always encoded first */
+    ARG_FLAG_FLAT_ARRAY
 };
 
 char *argname[] = {
@@ -54,16 +63,21 @@ char *argname[] = {
     "tokenpos", /* ARG_TOKENPOS */
     "token0",   /* ARG_TOKEN0 */
     "tagname",  /* ARG_TAGNAME */
+    "tag",      /* ARG_TAG */
     "attr",     /* ARG_ATTR */
+    "@attr",    /* ARG_ATTRARR */
     "attrseq",  /* ARG_ATTRSEQ */
     "text",     /* ARG_TEXT */
     "dtext",    /* ARG_DTEXT */
     "is_cdata", /* ARG_IS_CDATA */
     "offset",   /* ARG_OFFSET */
     "length",   /* ARG_LENGTH */
+    "line",     /* ARG_LINE */
+    "column",   /* ARG_COLUMN */
     "event",    /* ARG_EVENT */
     "undef",    /* ARG_UNDEF */
     /* ARG_LITERAL (not compared) */
+    /* ARG_FLAG_FLAT_ARRAY */
 };
 
 
@@ -93,7 +107,6 @@ report_event(PSTATE* p_state,
 	     event_id_t event,
 	     char *beg, char *end,
 	     token_pos_t *tokens, int num_tokens,
-	     STRLEN offset,
 	     SV* self
 	    )
 {
@@ -105,6 +118,11 @@ report_event(PSTATE* p_state,
     char *argspec;
     char *s;
 
+    /* capture offsets */
+    STRLEN offset = p_state->offset;
+    STRLEN line = p_state->line;
+    STRLEN column = p_state->column;
+
 #if 0
     {  /* used for debugging at some point */
 	char *s = beg;
@@ -118,6 +136,7 @@ report_event(PSTATE* p_state,
 	case E_END:         printf("END"); break;
 	case E_TEXT:        printf("TEXT"); break;
 	case E_PROCESS:     printf("PROCESS"); break;
+	case E_NONE:        printf("NONE"); break;
 	default:            printf("EVENT #%d", event); break;
 	}
 
@@ -140,11 +159,82 @@ report_event(PSTATE* p_state,
     }
 #endif
 
+    /* update offsets */
+    p_state->offset += end - beg;
+    if (line) {
+	char *s = beg;
+	char *nl = NULL;
+	while (s < end) {
+	    if (*s == '\n') {
+		p_state->line++;
+		nl = s;
+	    }
+	    s++;
+	}
+	if (nl)
+	    p_state->column = end - nl - 1;
+	else
+	    p_state->column += end - beg;
+    }
+
+    if (event == E_NONE)
+	return;
+    
 #ifdef MARKED_SECTION
     if (p_state->ms == MS_IGNORE)
 	return;
 #endif
 
+    /* tag filters */
+    if (p_state->ignore_tags || p_state->report_tags || p_state->ignore_elements) {
+
+	if (event == E_START || event == E_END) {
+	    SV* tagname = p_state->tmp;
+	    U32 hash;
+
+	    assert(num_tokens >= 1);
+	    sv_setpvn(tagname, tokens[0].beg, tokens[0].end - tokens[0].beg);
+	    if (!p_state->xml_mode)
+		sv_lower(aTHX_ tagname);
+
+	    if (p_state->ignoring_element) {
+		if (sv_eq(p_state->ignoring_element, tagname)) {
+		    if (event == E_START)
+			p_state->ignore_depth++;
+		    else if (--p_state->ignore_depth == 0) {
+			SvREFCNT_dec(p_state->ignoring_element);
+			p_state->ignoring_element = 0;
+		    }
+		}
+		return;
+	    }
+
+	    PERL_HASH(hash, SvPVX(tagname), SvCUR(tagname));
+
+	    if (p_state->ignore_elements &&
+		hv_fetch_ent(p_state->ignore_elements, tagname, 0, hash))
+	    {
+		p_state->ignoring_element = newSVsv(tagname);
+		p_state->ignore_depth = 1;
+		return;
+	    }
+
+	    if (p_state->ignore_tags &&
+		hv_fetch_ent(p_state->ignore_tags, tagname, 0, hash))
+	    {
+		return;
+	    }
+	    if (p_state->report_tags &&
+		!hv_fetch_ent(p_state->report_tags, tagname, 0, hash))
+	    {
+		return;
+	    }
+	}
+	else if (p_state->ignoring_element) {
+	    return;
+	}
+    }
+
     h = &p_state->handlers[event];
     if (!h->cb) {
 	/* event = E_DEFAULT; */
@@ -171,7 +261,9 @@ report_event(PSTATE* p_state,
 	}
 	else {
 	INIT_PEND_TEXT:
-	    p_state->pend_text_offset = p_state->chunk_offset + offset;
+	    p_state->pend_text_offset = offset;
+	    p_state->pend_text_line = line;
+	    p_state->pend_text_column = column;
 	    p_state->pend_text_is_cdata = p_state->is_cdata;
 	    sv_setpvn(p_state->pend_text, "", 0);
 	}
@@ -185,22 +277,33 @@ report_event(PSTATE* p_state,
 
     /* At this point we have decided to generate an event callback */
 
+    argspec = h->argspec ? SvPV(h->argspec, my_na) : "";
+
     if (SvTYPE(h->cb) == SVt_PVAV) {
-	/* start sub-array for accumulator array */
-	array = newAV();
+	
+	if (*argspec == ARG_FLAG_FLAT_ARRAY) {
+	    argspec++;
+	    array = (AV*)h->cb;
+	}
+	else {
+	    /* start sub-array for accumulator array */
+	    array = newAV();
+	}
     }
     else {
 	array = 0;
+	if (*argspec == ARG_FLAG_FLAT_ARRAY)
+	    argspec++;
+
 	/* start argument stack for callback */
 	ENTER;
 	SAVETMPS;
 	PUSHMARK(SP);
     }
 
-    argspec = h->argspec ? SvPV(h->argspec, my_na) : "";
-
     for (s = argspec; *s; s++) {
 	SV* arg = 0;
+	int push_arg = 1;
 	enum argcode argcode = (enum argcode)*s;
 
 	switch( argcode ) {
@@ -250,21 +353,34 @@ report_event(PSTATE* p_state,
 	    break;
 
 	case ARG_TOKEN0:
+	case ARG_TAGNAME:
 	    /* fall through */
 
-	case ARG_TAGNAME:
+	case ARG_TAG:
 	    if (num_tokens >= 1) {
 		arg = sv_2mortal(newSVpvn(tokens[0].beg,
 					  tokens[0].end - tokens[0].beg));
-		if (!p_state->xml_mode && argcode == ARG_TAGNAME)
+		if (!p_state->xml_mode && argcode != ARG_TOKEN0)
 		    sv_lower(aTHX_ arg);
+		if (argcode == ARG_TAG && event != E_START) {
+		    char *e_type = "!##/#?#";
+		    sv_insert(arg, 0, 0, &e_type[event], 1);
+		}
 	    }
 	    break;
 
 	case ARG_ATTR:
+	case ARG_ATTRARR:
 	    if (event == E_START) {
-		HV* hv = newHV();
+		HV* hv;
 		int i;
+		if (argcode == ARG_ATTR) {
+		    hv = newHV();
+		    arg = sv_2mortal(newRV_noinc((SV*)hv));
+		}
+		else
+		    push_arg = 0;  /* deal with argument pushing here */
+
 		for (i = 1; i < num_tokens; i += 2) {
 		    SV* attrname = newSVpvn(tokens[i].beg,
 					    tokens[i].end-tokens[i].beg);
@@ -289,15 +405,30 @@ report_event(PSTATE* p_state,
 
 		    if (!p_state->xml_mode)
 			sv_lower(aTHX_ attrname);
-		    if (!hv_store_ent(hv, attrname, attrval, 0)) {
-			SvREFCNT_dec(attrval);
+
+		    if (argcode == ARG_ATTR) {
+			if (!hv_store_ent(hv, attrname, attrval, 0)) {
+			    SvREFCNT_dec(attrval);
+			}
+			SvREFCNT_dec(attrname);
+		    }
+		    else { /* ARG_ATTRARR */
+			if (array) {
+			    av_push(array, attrname);
+			    av_push(array, attrval);
+			}
+			else {
+			    XPUSHs(sv_2mortal(attrname));
+			    XPUSHs(sv_2mortal(attrval));
+			}
 		    }
-		    SvREFCNT_dec(attrname);
 		}
-		arg = sv_2mortal(newRV_noinc((SV*)hv));
+	    }
+	    else if (argcode == ARG_ATTRARR) {
+		push_arg = 0;
 	    }
 	    break;
-      
+
 	case ARG_ATTRSEQ:       /* (v2 compatibility stuff) */
 	    if (event == E_START) {
 		AV* av = newAV();
@@ -332,13 +463,21 @@ report_event(PSTATE* p_state,
 	    break;
 
 	case ARG_OFFSET:
-	    arg = sv_2mortal(newSViv(p_state->chunk_offset + offset));
+	    arg = sv_2mortal(newSViv(offset));
 	    break;
 
 	case ARG_LENGTH:
 	    arg = sv_2mortal(newSViv(end - beg));
 	    break;
 
+	case ARG_LINE:
+	    arg = sv_2mortal(newSViv(line));
+	    break;
+
+	case ARG_COLUMN:
+	    arg = sv_2mortal(newSViv(column));
+	    break;
+
 	case ARG_EVENT:
 	    assert(event >= 0 && event < EVENT_COUNT);
 	    arg = sv_2mortal(newSVpv(event_id_str[event], 0));
@@ -361,22 +500,25 @@ report_event(PSTATE* p_state,
 	    break;
 	}
 
-	if (!arg)
-	    arg = sv_mortalcopy(&PL_sv_undef);
+	if (push_arg) {
+	    if (!arg)
+		arg = sv_mortalcopy(&PL_sv_undef);
 
-	if (array) {
-	    /* have to fix mortality here or add mortality to
-             * XPUSHs after removing it from the switch cases.
-             */
-	    av_push(array, SvREFCNT_inc(arg));
-	}
-	else {
-	    XPUSHs(arg);
+	    if (array) {
+		/* have to fix mortality here or add mortality to
+		 * XPUSHs after removing it from the switch cases.
+		 */
+		av_push(array, SvREFCNT_inc(arg));
+	    }
+	    else {
+		XPUSHs(arg);
+	    }
 	}
     }
 
     if (array) {
-	av_push((AV*)h->cb, newRV_noinc((SV*)array));
+	if (array != (AV*)h->cb)
+	    av_push((AV*)h->cb, newRV_noinc((SV*)array));
     }
     else {
 	PUTBACK;
@@ -396,7 +538,7 @@ report_event(PSTATE* p_state,
 
 
 EXTERN SV*
-argspec_compile(SV* src)
+argspec_compile(SV* src, PSTATE* p_state)
 {
     dTHX;
     SV* argspec = newSVpvn("", 0);
@@ -406,8 +548,23 @@ argspec_compile(SV* src)
 
     while (isHSPACE(*s))
 	s++;
+
+    if (*s == '@') {
+	/* try to deal with '@{ ... }' wrapping */
+	char *tmp = s + 1;
+	while (isHSPACE(*tmp))
+	    tmp++;
+	if (*tmp == '{') {
+	    char c = ARG_FLAG_FLAT_ARRAY;
+	    sv_catpvn(argspec, &c, 1);
+	    tmp++;
+	    while (isHSPACE(*tmp))
+		tmp++;
+	    s = tmp;
+	}
+    }
     while (s < end) {
-	if (isHNAME_FIRST(*s)) {
+	if (isHNAME_FIRST(*s) || *s == '@') {
 	    char *name = s;
 	    int a = ARG_SELF;
 	    char temp;
@@ -425,7 +582,13 @@ argspec_compile(SV* src)
 		    break;
 	    }
 	    if (a < ARG_LITERAL) {
-		sv_catpvf(argspec, "%c", (unsigned char) a);
+		char c = (unsigned char) a;
+		sv_catpvn(argspec, &c, 1);
+
+		if (a == ARG_LINE || a == ARG_COLUMN) {
+		    if (!p_state->line)
+			p_state->line = 1; /* enable tracing of line/column */
+		}
 	    }
 	    else {
 		croak("Unrecognized identifier %s in argspec", name);
@@ -440,9 +603,12 @@ argspec_compile(SV* src)
 	    if (*s == *string_beg) {
 		/* literal */
 		int len = s - string_beg - 1;
+		unsigned char buf[2];
 		if (len > 255)
 		    croak("Literal string is longer than 255 chars in argspec");
-		sv_catpvf(argspec, "%c%c", ARG_LITERAL, len);
+		buf[0] = ARG_LITERAL;
+		buf[1] = len;
+		sv_catpvn(argspec, (char*)buf, 2);
 		sv_catpvn(argspec, string_beg+1, len);
 		s++;
 	    }
@@ -459,6 +625,16 @@ argspec_compile(SV* src)
 
 	while (isHSPACE(*s))
 	    s++;
+	
+	if (*s == '}' && SvPVX(argspec)[0] == ARG_FLAG_FLAT_ARRAY) {
+	    /* end of '@{ ... }' */
+	    s++;
+	    while (isHSPACE(*s))
+		s++;
+	    if (s < end)
+		croak("Bad argspec: stuff after @{...} (%s)", s);
+	}
+
 	if (s == end)
 	    break;
 	if (*s != ',') {
@@ -479,30 +655,35 @@ flush_pending_text(PSTATE* p_state, SV* self)
     bool   old_unbroken_text = p_state->unbroken_text;
     SV*    old_pend_text     = p_state->pend_text;
     bool   old_is_cdata      = p_state->is_cdata;
-    STRLEN old_chunk_offset  = p_state->chunk_offset;
+    STRLEN old_offset        = p_state->offset;
+    STRLEN old_line          = p_state->line;
+    STRLEN old_column        = p_state->column;
 
     assert(p_state->pend_text && SvOK(p_state->pend_text));
 
     p_state->unbroken_text = 0;
     p_state->pend_text     = 0;
     p_state->is_cdata      = p_state->pend_text_is_cdata;
-    p_state->chunk_offset  = p_state->pend_text_offset;
+    p_state->offset        = p_state->pend_text_offset;
+    p_state->line          = p_state->pend_text_line;
+    p_state->column        = p_state->pend_text_column;
 
     report_event(p_state, E_TEXT,
 		 SvPVX(old_pend_text), SvEND(old_pend_text),
-		 0, 0, 0,
-		 self);
+		 0, 0, self);
     SvOK_off(old_pend_text);
 
     p_state->unbroken_text = old_unbroken_text;
     p_state->pend_text     = old_pend_text;
     p_state->is_cdata      = old_is_cdata;
-    p_state->chunk_offset  = old_chunk_offset;
+    p_state->offset        = old_offset;
+    p_state->line          = old_line;
+    p_state->column        = old_column;
 }
 
 
 static char*
-parse_comment(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
+parse_comment(PSTATE* p_state, char *beg, char *end, SV* self)
 {
     char *s = beg;
 
@@ -531,7 +712,7 @@ parse_comment(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 		report_event(p_state, E_COMMENT,
 			     beg - 4, s,
 			     tokens, num_tokens,
-			     offset, self);
+			     self);
 		FREE_TOKENS;
 
 		return s;
@@ -576,7 +757,7 @@ parse_comment(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 		    s++;
 		    /* yup */
 		    report_event(p_state, E_COMMENT, beg-4, s, &token_pos, 1,
-				 offset, self);
+				 self);
 		    return s;
 		}
 	    }
@@ -710,6 +891,7 @@ FIND_NAMES:
 	    p_state->ms_stack = newAV();
 	av_push(p_state->ms_stack, newRV_noinc((SV*)tokens));
 	marked_section_update(p_state);
+	report_event(p_state, E_NONE, beg, s, 0, 0, self);
 	return s;
     }
 
@@ -725,7 +907,7 @@ PREMATURE:
 
 
 static char*
-parse_decl(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
+parse_decl(PSTATE* p_state, char *beg, char *end, SV* self)
 {
     char *s = beg + 2;
 
@@ -743,7 +925,7 @@ parse_decl(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 	/* yes, two dashes seen */
 	s++;
 
-	tmp = parse_comment(p_state, s, end, offset, self);
+	tmp = parse_comment(p_state, s, end, self);
 	return (tmp == s) ? beg : tmp;
     }
 
@@ -763,7 +945,7 @@ parse_decl(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 	empty.beg = s;
 	empty.end = s;
 	s++;
-	report_event(p_state, E_COMMENT, beg, s, &empty, 1, offset, self);
+	report_event(p_state, E_COMMENT, beg, s, &empty, 1, self);
 	return s;
     }
 
@@ -854,8 +1036,7 @@ parse_decl(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 	    goto PREMATURE;
 	if (*s == '>') {
 	    s++;
-	    report_event(p_state, E_DECLARATION, beg, s, tokens, num_tokens,
-			 offset, self);
+	    report_event(p_state, E_DECLARATION, beg, s, tokens, num_tokens, self);
 	    FREE_TOKENS;
 	    return s;
 	}
@@ -874,7 +1055,7 @@ parse_decl(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 
 
 static char*
-parse_start(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
+parse_start(PSTATE* p_state, char *beg, char *end, SV* self)
 {
     char *s = beg;
     SV* attr;
@@ -976,10 +1157,9 @@ parse_start(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
     if (*s == '>') {
 	s++;
 	/* done */
-	report_event(p_state, E_START, beg, s, tokens, num_tokens, offset, self);
+	report_event(p_state, E_START, beg, s, tokens, num_tokens, self);
 	if (empty_tag)
-	    report_event(p_state, E_END, s, s, tokens, 1,
-			 offset + (s - beg), self);
+	    report_event(p_state, E_END, s, s, tokens, 1, self);
 
 	if (!p_state->xml_mode) {
 	    /* find out if this start tag should put us into literal_mode
@@ -1001,7 +1181,7 @@ parse_start(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 			if (!--len) {
 			    /* found it */
 			    p_state->literal_mode = literal_mode_elem[i].str;
-			    p_state->is_cdata = 1;
+			    p_state->is_cdata = literal_mode_elem[i].is_cdata;
 			    /* printf("Found %s\n", p_state->literal_mode); */
 			    goto END_OF_LITERAL_SEARCH;
 			}
@@ -1026,7 +1206,7 @@ PREMATURE:
 
 
 static char*
-parse_end(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
+parse_end(PSTATE* p_state, char *beg, char *end, SV* self)
 {
     char *s = beg+2;
     hctype_t name_first, name_char;
@@ -1052,7 +1232,7 @@ parse_end(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 	    if (*s == '>') {
 		s++;
 		/* a complete end tag has been recognized */
-		report_event(p_state, E_END, beg, s, &tagname, 1, offset, self);
+		report_event(p_state, E_END, beg, s, &tagname, 1, self);
 		return s;
 	    }
 	}
@@ -1065,8 +1245,7 @@ parse_end(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
 
 
 static char*
-parse_process(PSTATE* p_state, char *beg, char *end,
-	      STRLEN offset, SV* self)
+parse_process(PSTATE* p_state, char *beg, char *end, SV* self)
 {
     char *s = beg + 2;  /* skip '<?' */
     /* processing instruction */
@@ -1087,7 +1266,7 @@ parse_process(PSTATE* p_state, char *beg, char *end,
       
 	    /* a complete processing instruction seen */
 	    report_event(p_state, E_PROCESS, beg, s,
-			 &token_pos, 1, offset, self);
+			 &token_pos, 1, self);
 	    return s;
 	}
 	s++;
@@ -1097,7 +1276,7 @@ parse_process(PSTATE* p_state, char *beg, char *end,
 
 
 static char*
-parse_null(PSTATE* p_state, char *beg, char *end, STRLEN offset, SV* self)
+parse_null(PSTATE* p_state, char *beg, char *end, SV* self)
 {
     return 0;
 }
@@ -1116,19 +1295,24 @@ parse(pTHX_
     STRLEN len;
 
     if (!chunk) {
-	/* flush */
+	/* eof */
 	if (p_state->buf && SvOK(p_state->buf)) {
 	    /* flush it */
 	    STRLEN len;
 	    char *s = SvPV(p_state->buf, len);
 	    assert(len);
-	    report_event(p_state, E_TEXT, s, s+len, 0, 0, 0, self);
-	    p_state->chunk_offset += len;
+	    report_event(p_state, E_TEXT, s, s+len, 0, 0, self);
 	    SvREFCNT_dec(p_state->buf);
 	    p_state->buf = 0;
 	}
 	if (p_state->pend_text && SvOK(p_state->pend_text))
 	    flush_pending_text(p_state, self);
+
+	if (p_state->ignoring_element) {
+	    /* document not balanced */
+	    SvREFCNT_dec(p_state->ignoring_element);
+	    p_state->ignoring_element = 0;
+	}
 	return;
     }
 
@@ -1189,9 +1373,9 @@ parse(pTHX_
 			s++;
 			if (t != end_text)
 			    report_event(p_state, E_TEXT, t, end_text,
-					 0, 0, t - beg, self);
+					 0, 0, self);
 			report_event(p_state, E_END,  end_text, s,
-				     &end_token, 1, end_text - beg, self);
+				     &end_token, 1, self);
 			p_state->literal_mode = 0;
 			p_state->is_cdata = 0;
 			t = s;
@@ -1214,7 +1398,8 @@ parse(pTHX_
 			/* marked section end */
 			if (t != end_text)
 			    report_event(p_state, E_TEXT, t, end_text,
-					 0, 0, t - beg, self);
+					 0, 0, self);
+			report_event(p_state, E_NONE, end_text, s, 0, 0, self);
 			t = s;
 			SvREFCNT_dec(av_pop(p_state->ms_stack));
 			marked_section_update(p_state);
@@ -1240,10 +1425,12 @@ parse(pTHX_
 		    if (*s == '>') {
 			s++;
 			report_event(p_state, E_TEXT, t, end_text,
-				     0, 0, t - beg, self);
+				     0, 0, self);
+			report_event(p_state, E_NONE, end_text, s,
+				     0, 0, self);
+			t = s;
 			SvREFCNT_dec(av_pop(p_state->ms_stack));
 			marked_section_update(p_state);    
-			t = s;
 			continue;
 		    }
 		}
@@ -1253,7 +1440,7 @@ parse(pTHX_
 	}
 	if (s != t) {
 	    if (*s == '<') {
-		report_event(p_state, E_TEXT, t, s, 0, 0, t - beg, self);
+		report_event(p_state, E_TEXT, t, s, 0, 0, self);
 		t = s;
 	    }
 	    else {
@@ -1272,7 +1459,7 @@ parse(pTHX_
 		}
 		s++;
 		if (s != t)
-		    report_event(p_state, E_TEXT, t, s, 0, 0, t - beg, self);
+		    report_event(p_state, E_TEXT, t, s, 0, 0, self);
 		break;
 	    }
 	}
@@ -1283,7 +1470,7 @@ parse(pTHX_
 	/* next char is known to be '<' and pointed to by 't' as well as 's' */
 	s++;
 
-	if ( (new_pos = parsefunc[(unsigned char)*s](p_state, t, end, t - beg, self))) {
+	if ( (new_pos = parsefunc[(unsigned char)*s](p_state, t, end, self))) {
 	    if (new_pos == t) {
 		/* no progress, need more data to know what it is */
 		s = t;
@@ -1300,8 +1487,6 @@ parse(pTHX_
 
 DONE:
 
-    p_state->chunk_offset += (s - beg);
-
     if (s == end || p_state->eof) {
 	if (p_state->buf) {
 	    SvOK_off(p_state->buf);
